using System;
using System.Collections;

public class PriorityQueue
{
    // Die an Push bergebenen Daten werden zusammen mit ihrer Prioritt in Objekte der
    // Klasse Item verpack und so intern gespeichert
    public class Item
    {
        public Object obj;
        public int priority;

        public Item(Object obj, int priority)
        {
            this.obj = obj;
            this.priority = priority;
        }
    }

    // Intern werden die Daten in einem einfachen Array verwahrt
    private Item[] data;
    private int size;
    private const int blocksize = 10;

    public PriorityQueue()
    {
        data = new Item[blocksize];
        size = 0;
    }

    // Liefere das vorderste Datenelement zurck und lsche dieses gleichzeitig aus der 
    // Datenstruktur. Das das Lschen am effizientesten am Ende des Arrays ausgefhrt 
    // werden kann, verwenden wir das hintere Ende des Arrays als vorderes Ende der 
    // PriorityQueue
    public Object Pop()
    {
        if (size == 0)
            return null;

        Object obj = data[--size].obj;
        data[size] = null;
        return obj;
    }

    // Datenelement einfgen
    public void Push(Object obj, int priority)
    {
        // internes Array bei Bedarf vergrern
        if (size == data.Length)
        {
            Item[] newData = new Item[data.Length + blocksize];
            Array.Copy(data, 0, newData, 0, size);
            data = newData;
        }

        if (size == 0)
        {
            data[0] = new Item(obj, priority);
            ++size;
        }
        else
        {
            // Einfgeposition suchen
            int insertpos = size;

            while (insertpos > 0)
            {
                if (data[insertpos - 1].priority < priority)
                    break;

                --insertpos;
            }

            Array.Copy(data, insertpos, data, insertpos + 1, size - insertpos);
            data[insertpos] = new Item(obj, priority);
            ++size;
        }
    }

    // Enumerator zurckliefern
    public IEnumerable FrontToEnd()
    {
        for (int i = size - 1; i >= 0; i--)
            yield return data[i];
    }
    public IEnumerable EndToFront()
    {
        for (int i = 0; i < size; i++)
            yield return data[i];
    }
}

public class Program
{
    static void Main()
    {
        PriorityQueue pq = new PriorityQueue();

        pq.Push(10, 2);
        pq.Push(444, 3);
        pq.Push(123456, 6);
        pq.Push(333, 3);
        pq.Push(5, 1);
        pq.Push(11, 2);
        pq.Push(12, 2);
        pq.Push(12345, 5);
        pq.Push(0, 0);
        pq.Push(123456789, 9);

        Console.WriteLine("\n Aktueller Inhalt der PriorityQueue \n");
        foreach (PriorityQueue.Item i in pq.FrontToEnd())
        {
            if (i != null)
                Console.WriteLine("{0,10} {1}", i.obj, i.priority);
        }

        Console.WriteLine();

        foreach (PriorityQueue.Item i in pq.EndToFront())
        {
            if (i != null)
                Console.WriteLine("{0,10} {1}", i.obj, i.priority);
        }

    }
}

