<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html" />

<!-- Parameter 
     $TheFullType wird der Name der Klasse zugewiesen (muss im ersten <member>-Tag beschrieben sein)
     -->
   <xsl:param name="TheFullType" select="substring-after(./doc/members/member[1]/@name, ':')" />

   <!-- Template fr die oberste Ebene
     gibt das HTML-Grundgerst aus und ruft intern die Templates fr die Berschrift und die einzelnen Member auf 
     -->
<xsl:template match="doc">
<html>
   <head>
      <style type="text/css">
         h1 { background-color : aqua;
              text-align : center}
         h2 { background-color : rgb(200, 200, 200);
              margin-top : 50 }
         h3 { margin-top : 20 }
         th { background-color: rgb(200, 200, 200) }
         td { background-color: rgb(240, 240, 240);
              padding-left : 5 }
         thead { font-size: 14; font-weight : bold; }
     </style>       
   </head>

   <body>
      <xsl:apply-templates select="./assembly/name" />
	   <xsl:apply-templates select="./members/member[position()>1]" />
   </body>
          
</html>
</xsl:template>
           
   <!-- Templates der zweiten Ebene -->
   <xsl:template match="name">
      <h1>
	      <xsl:text>Assembly: </xsl:text>
	      <xsl:value-of select="." />
	      <br />
		   <xsl:text>Typ: </xsl:text>
			<xsl:value-of select="$TheFullType" />
		</h1>

      <xsl:apply-templates select="../../members/member[1]/summary" />
      <xsl:apply-templates select="../../members/member[1]/list[@type='table']" />
      <xsl:apply-templates select="../../members/member[1]/remarks" />
   </xsl:template>
	
		
	<xsl:template match="member">
       <!-- Parameter 
            $TheMember speichert den Namen des aktuellen Elements 
            -->
       <xsl:param name="TheMember" select="substring-after(./@name, ':')" />

       <!-- Member-Name als berschrift zweiter Ebene ausgeben -->
       <xsl:choose>
           <xsl:when test="starts-with(@name,'T:')">
               <h2>
                   Typ <xsl:value-of select="$TheMember" />
               </h2>

               <!-- Fr den Typ gebe aus : -->
               <xsl:apply-templates select="summary" />
               <xsl:apply-templates select="remarks" />
          </xsl:when>

           <xsl:when test="starts-with(@name,'F:')">
               <h2>
                   Feld <a id="{$TheMember}"><xsl:value-of select="$TheMember" /></a>
               </h2>

               <!-- Fr ein Feld gebe aus : -->
               <xsl:apply-templates select="summary" />
           </xsl:when>

           <xsl:when test="(starts-with(@name,'M:')) and (contains(@name, '#'))">
               <h2>
                   Konstruktor <a id="{$TheMember}"><xsl:value-of select="concat($TheFullType, substring-after($TheMember, '#ctor'))" /></a>
               </h2>

               <!-- Fr eine Methode gebe aus : -->
               <xsl:apply-templates select="summary" />
               <xsl:apply-templates select="param" />
               <xsl:apply-templates select="exception" />
               <xsl:apply-templates select="remarks" />
           </xsl:when>

           <xsl:when test="(starts-with(@name,'M:')) and not (contains(@name, '#'))">
               <h2>
                   Methode <a id="{$TheMember}"><xsl:value-of select="$TheMember" /></a>
               </h2>

               <!-- Fr eine Methode gebe aus : -->
               <xsl:apply-templates select="summary" />
               <xsl:apply-templates select="param" />
               <xsl:apply-templates select="returns" />
               <xsl:apply-templates select="exception" />
               <xsl:apply-templates select="remarks" />
           </xsl:when>
            
           <xsl:when test="starts-with(@name,'P:')">
               <h2>
                   Eigenschaft <a id="{$TheMember}"><xsl:value-of select="$TheMember" /></a>
               </h2>

               <!-- Fr eine Eigenschaft gebe aus : -->
               <xsl:apply-templates select="summary" />
               <xsl:apply-templates select="value" />
               <xsl:apply-templates select="remarks" />
           </xsl:when>
            
           <xsl:when test="starts-with(@name,'E:')">
               <h2>
                   Ereignis <a id="{$TheMember}"><xsl:value-of select="$TheMember" /></a>
               </h2>
                
               <!-- Fr ein Feld gebe aus : -->
               <xsl:apply-templates select="summary" />
           </xsl:when>
	   </xsl:choose>
   </xsl:template>



   <!-- Templates der dritten Ebene -->
   <xsl:template match="summary">
      <p>
         <xsl:apply-templates />
      </p>    
   </xsl:template>

   <xsl:template match="remarks">
      <h3>Hinweis</h3>
      <p>
         <xsl:apply-templates />
      </p>
   </xsl:template>

	<xsl:template match="param">
      <xsl:if test="position() = 1">
         <h3>Parameter</h3>
      </xsl:if>
      <p>
         <i>
            <xsl:value-of select="@name" />
         </i>
      </p>
      <p>
         <xsl:apply-templates />
      </p>
	</xsl:template>
	
	<xsl:template match="returns">
      <h3>Rckgabewert</h3>
      <p>
         <xsl:apply-templates />
      </p>
	</xsl:template>
	
	<xsl:template match="exception">
      <xsl:if test="position() = 1">
         <h3>Ausnahmen</h3>
      </xsl:if>
      <p>
         <i>
            <xsl:value-of select="substring-after(@cref, ':')" />
         </i>
      </p>
      <p>
         <xsl:apply-templates />
      </p>
   </xsl:template>

   <xsl:template match="value">
      <h3>Eigenschaftenwert</h3>
      <p>
         <xsl:apply-templates />
      </p>
   </xsl:template>
	


   <!-- Sonstige Templates -->
	<xsl:template match="list[@type='table']">
	   <h3>
	      <xsl:value-of select="listheader" />
	   </h3>
  
	   <table border="1" width="100%">
         <tr>
            <th width="400">Name</th>
            <th>Beschreibung</th>
         </tr>
            <xsl:apply-templates select="item" />
	   </table>
	</xsl:template>

	<xsl:template match="item">
	   <tr>
         <td>
            <xsl:apply-templates select="./term" />
			   <!-- <code><xsl:value-of select="name" /></code> -->
			</td>
			<td>
            <xsl:apply-templates select="./description" />
			   <!-- <xsl:value-of select="description" /> -->
			</td>
	   </tr>
   </xsl:template>

   <xsl:template match="item/term">
      <code>
         <xsl:apply-templates />
      </code>
   </xsl:template>

   <xsl:template match="item/description">
         <xsl:apply-templates />
   </xsl:template>



   <xsl:template match="see">
      <xsl:variable name="ref" select="substring-after(@cref,':') " />
      <a href="#{$ref}"><xsl:value-of select="$ref"/></a>
   </xsl:template>

   <xsl:template match="c">
      <code>
         <xsl:apply-templates />
      </code>
   </xsl:template>

   <xsl:template match="code">
      <pre>
         <xsl:apply-templates />
      </pre>
   </xsl:template>

   <xsl:template match="para">
		<p>
			<xsl:apply-templates />
		</p>
	</xsl:template>
	
</xsl:stylesheet>
